; README of mp_absredd_Ebv, ps=ps, no_mpfit=no_mpfit
;  
; ----------------------------------------------------------------- 
; IDL Program: mp_absredd_Ebv - Modification of mp_absredd for stars with unknow Av (from Gaia.
; Assuming that 50% of the dust mass is in dark dust and derived Nl,ND fomr E(B-V)
; fits absolute Reddening curve. Start parameters form Siebenmorgen 2023 of the general ISM 
;
; Externals:
;   a)   ./MPFIT/mpfit.pro : MPFIT IDL fitting package
;   b.i)  fkt_absreddRad   : External function of mpfit() -- radii free
;   b.ii) fkt_absredd      : External function of mpfit() -- radii fixed
;   c)   ./a.crossec_dd  : Executable of crossec_dd.f (called by fkt_absredd*pro)
;
; 
; INPUT:
;   1) File './Input/mk_Tab1.xdr', which includes:
;      Name of the stars, and Rv_ref, Av_ref, Ebv_ref from the reddening curves
;      inspected by Siebenmorgen et al,2023, A&A, 676, 132.
;
;   2) Files:
;      2.1) ./StartParameters/HD129557_PAH2170.wq
;           Drude Parameters x0, gam for the 2175AA bump used for
;           computing PAH cross sections. Use parameters in the second
;           line, which are derived experimenting with a.crossec_dd (first line
;           contains parameters derived from spline fits of the reddening curve paper).
;
;      2.2) ./StartParameters/jsm12fit.inpStart
;           Start (12) parameters of dust model read by a.crossec_dd when fitting the reddening curve. One may
;           use default parameters as in crossec_dd.f, .... 
;
;      2.3) ./StartParameters/HD*para.xdr
;           IDL saveset includes previous parameters (2.1 and 2.2) as
;           well as the relative reddening curve E(w-V)/E(B-V) of the
;           star that shall be fit. (Variables:
;           wdata, data, with rms error edata; 
;           Rv_ref, E(B-V)=ebv of the reddening curve paper and
;           Rv=Rv_mod of the Dark Dust model fit (returned by a.crossec_dd)
;           and reduced chi2 (divided by dot).
;
; OUTPUT:
;   './ResultEbv/'+ target+'_Kappa.out'    : Abs, sca cross sections K
;   './ResultEbv/'+ target+'_PolKappa.out' : Polarization
;   './ResultEbv/'+ target+'_PAH2170.wq'   : Update of 2.1
;   './ResultEbv/'+ target+'_jsm12fit.inp' : Update of 2.2
;   './ResultEbv/'+ target+'_para.xdr'     : Update of 2.3
;   './ResultEbv/'+ target+'_message.out'  : Messages of final a.crossec_dd 
;   idl.ps : PS file showing reddening curves (data and dark dust fit)
;
; Keywords:
;   ps: plot reddening curve and fit to pl_allRedd.pdf
;   no_mpfit: 1=nofit 0=mpfit is used (default)
;
; Tips:
;   a) In case you wish to fit a specific star, uncomment the target name setting in
;      the " for i = 0, nn-1 do begin" loop below.
;   b) Parallelization is useful when starting fitting a larger sample.
;      The vectorization of the code can be done by separating the
;      number of stars into different threads. This shall be adjusted to
;      your number of available IDL licenses.
;
; Example: in IDL enter:
;   !PATH = './AAstron/:' + !PATH
;   !PATH = './Coyote/:'  + !PATH
;   !PATH = './MPFIT/:'  + !PATH
;   mp_absredd_Ebv, /ps
;
; ---------------------------------------------------
;


; ===================================================================================================
Function fkt_absredd, wdata, para

; help function for mpfit, 9 fit parameters of dark dust (DD) model: 
; 0     1       2        3         4     5     6     7               8
; abuc, abuvsi, abucvgr, abucpahs, qmrn alec, alesi, arad_polmin_aC, arad_polmin_Si
; 1) For alec,alesi, arad_polmin* fixed parameters
; 2) For alec,alesi, arad_polmin* feee  parameters (interpol.)
; ------------------------------------------
;
; Get parain Check which parameter is free or constant
restore, './Input/tmp_parain.xdr'

; Polarisation data
;   readcol, './Data/poldata.tab',  wpd, Int, dInt, Qint, dQint, nQ, $
;         Uint, dUint, nU, Pd, epd,theta, dtheta, skip=0, /sil
;     npd = n_elements(pd)   

;
; reads previous parameters: alec and alesi
readcol, './Input/jsm12fit.inp', /sil, skipline=1, numli=1,  $
         abuc, abusi, abuvsi, abucvgr, abucpahs, $
         qmrn, alec,  alesi, arad_polmin_aC, arad_polmin_Si, arad_polmax, arad_Darkmax 
 qmrn    = qmrn(0)  > 2.
 qmrn    = qmrn(0) < 4.
 
abusi          = abusi(0) *1.d0
para           = para     *1d0
abuc           = para(0)  
abuvsi         = para(1) 
abucvgr        = para(2) 
abucpahs       = para(3) 
qmrn           = para(4)
alec           = para(5) 
alesi          = para(6)
arad_polmin_aC = para(7) < alec/1.05
arad_polmin_Si = para(8) < alesi/1.05


;abundance constraints [C]/[Si] < 5.25
if  para(0)        gt 5.25*(abusi + abuvsi) -abucvgr - abucpahs then begin
   print, 'abundance constraint not ok in absredd '
   print, '                          reset para(0) = ', abuc
    abuc    = abuc < 5.25*(abusi + abuvsi) -abucvgr - abucpahs
    para(0) = abuc
    print, '                          to    para(0) = ', para(0)
    
 endif


   x0             =   para(9)
   gam            =   para(10)   



; use org Drude parameters not those form redd fit !
   get_lun, iu
   openw,iu,'./Input/PAH2170.wq'
   printf, iu, 'x0 (mu)    gam : Drude Parameters at 2175AA bump as in reddening papers'
   printf,iu, x0, gam          ; used here considerung PAH 
   close, iu
   free_lun, iu


   
arad_polmax    = arad_polmax(0)
arad_Darkmax   = arad_Darkmax(0)
alec           = alec(0)
alesi          = alesi(0)
arad_polmin_aC = arad_polmin_aC(0)
arad_polmin_Si = arad_polmin_Si(0)

; set up radii:
narad          = 130
rr             = 6.e-7* 1.05^ findgen(narad)
alec           = alec            > min(rr)
alec           = alec            < max(rr)
alesi          = alesi           < max(rr)
alesi          = alesi           > min(rr)
arad_polmin_aC = arad_polmin_aC  > min(rr)
arad_polmin_Si = arad_polmin_Si  > min(rr)
arad_polmin_aC = arad_polmin_aC  < alec/1.05
arad_polmin_Si = arad_polmin_Si  < alesi/1.05

;
para(0) = para(0) 
para(5) = alec        
para(6) = alesi       
para(7) = arad_polmin_aC
para(8) = arad_polmin_Si
; check arad_polmin_* le alesi:
if para(7) ge para(5)/1.049  or para(8) ge para(6)/1.049 then begin
   print, ' arad_polmin_aC, alec '
   print,   arad_polmin_aC, alec
   print, ' arad_polmin_Si, alesi '
   print,   arad_polmin_Si, alesi
   print,   para(5:8)
   stop, ' arad_polmin* gt alec or alesisi ?'
endif

;
; ---------------------------------------------
; alec, alesi as fixed parameters:

 get_lun, iu
  openw,iu,'./Input/jsm12fit.inp'
printf, iu, '  abuc    abusi    avsi   avgr    apah     qmrn  rlec      rlesi     r_pmin_aC r_pmin_Si r_pmax    r_Darkmax '
printf, iu, format='(6f8.3, 6e10.3)', abuc, abusi, abuvsi, abucvgr, abucpahs, qmrn, $
        alec, alesi, arad_polmin_aC, arad_polmin_Si, arad_polmax, arad_Darkmax 
  close, iu
free_lun, iu

spawn,   './a.crossec_dd >  message.out'

; Reddening curve of model for radii alec, alesi
 readcol, './Output/tau4fit.out', wr, taum, reddm, absredd, skipline=1, /sil



; for unknow Dgaia the first 2 mpfit iterations only include data up
; to the K band and not E(oo)= -1.08574 * tauv, to ensure set :
  if wdata(0) lt 2.16 then absredd = absredd(1:*)
 
 yfit = [absredd]               ; only redd fit

  return, yfit




end

; ===================================================================================================

Function fkt_absreddrad, wdata, parad

; RSI 28.11.2023
; help function for mpfit, 3 raddi as fit parameters: lec, lesi, led to fit absulte reddening
; ------------------------------------------
;  
;
; reads previous parameters: alec and alesi
 readcol, './Input/jsm12fit.inp', /sil, skipline=1, numli=1,  $
  abuc, abusi, abuvsi, abucvgr, abucpahs, qmrn, alec,  alesi, arad_polmin_aC, arad_polmin_Si, arad_polmax, aled

 qmrn    = qmrn(0)  > 2.
 qmrn    = qmrn(0)  < 4.


  narad             = 111
  rarad             = 1.502e-5 * 1.05^ findgen(narad)  < 0.010375
 lecsimax           = 17   ; entspricht max grainn size in d.Q of 3.2784775e-5
 
lec      = (nint(parad(0)) < lecsimax)  > 0
lesi     = (nint(parad(1)) < lecsimax)  > 0
led      = (nint(parad(2)) < (narad-1)) > 0


alec     = rarad(lec)    < 3.28e-5 
alesi    = rarad(lesi)   < 3.28e-5 
aled     = rarad(led)    < 3.10d-4
aled     = aled          < 3.10d-4


;print, format = '(a25, i4,1e9.2,4x, i4,1e9.2,4x, i4,1e9.2)', 'fkt_jsmRad: aC,Si,Dark : ', lec, alec, lesi, alesi, led, aled

arad_polmax  = aled

 get_lun, iu
  openw,iu,'./Input/jsm12fit.inp'
printf, iu, '  abuc    abusi    avsi   avgr    apah     qmrn  rlec      rlesi     r_pmin_aC r_pmin_Si r_pmax    r_Darkmax '
printf, iu, format='(6f8.3, 6e10.3)', abuc, abusi, abuvsi, abucvgr, abucpahs, qmrn, $
        alec, alesi, arad_polmin_aC, arad_polmin_Si, arad_polmax, aled
  close, iu
free_lun, iu


spawn,   './a.crossec_dd >  message.out'

; Reddening curve of model for radii alec, alesi
readcol, './Output/tau4fit.out', wr, taum, reddm, absredd, skipline=1, /sil


; for unknow Dgaia the first 2 mpfit iterations only include data up
; to the K band and not E(oo)= -1.08574 * tauv, to ensure set :
  if wdata(0) lt 2.16 then absredd = absredd(1:*)

yfit = [absredd]                  ; absolure  redd E(w-V) fit



  return, yfit
end

; ===================================================================================================

Pro mp_absredd_Ebv, ps=ps, no_mpfit=no_mpfit
  if not keyword_set(no_mpfit) then      no_mpfit = 0

window, 0, xsi=700, ysiz=700
   !p.thick=2
   !p.charsize =1.3
   !P.font = 7
 if keyword_set(ps) then begin
    !P.font = 0
    a = PSWINDOW(/cm)
    device, decomposed=0
    set_plot, 'PS'
    device, filename='idl.ps',BITS_PER_PIXEL=8, /color,  $
           XSIZE=a.xsize, YSIZE=a.ysize, XOFFSET=a.xoffset, YOFFSET=a.yoffset
 endif
  narad             = 111
  rarad             = 1.502e-5 * 1.05^ findgen(narad)  < 3.10d-4

  redchi2     = 999.

; PAH Drude parameters:
  x0org  = 4.6
  gamorg = 1.0

; restore Tab.1 entries
  restore, /verb, './Input/mk_Tab1.xdr'
   vrv_ref  = rv                ; Rv  of Extinction paper
   aV_ref   = av   > 0.001      ; Av  of Extinction paper
   delvar,  av, rv

   
; ---------------------------  Fit over all stars :
;
   
nn    = n_elements(star)
nn2   = 0 & nn3 = 0

for i = 0, nn-1 do begin
   
; target  = 'HD027778'         ; just use one star
 target  = star(i)              ; if you wish to fit all stars

   if target eq star(i) then begin
 print
 print, ' ============= START target    =   ', target, '      ================ '
 print

;  finp = findfile('./StartParameters/'+target+'_jsm12fit.inp', cou=nfi)
  finp = findfile('./Input/jsm12fit.inpStart', cou=nfi)
  fpah = findfile('./StartParameters/'+target+'_PAH2170.wq',   cou=nfp)
  fxdr = findfile('./StartParameters/'+target+'_para.xdr',   cou=nf)
  
  
     if nf ne 1 or nfi ne 1 or nfp ne 1 then $
     stop, 'check files =', fext, finp, fpah, fxdr
     finp = finp(0) & fpah = fpah(0) & fxdr = fxdr(0)
     
; cp input files:     
                  comm = 'cp ' +finp(0)  + ' ./Input/jsm12fit.inp' 
           spawn, comm 
; print, '1:   ', comm
                  comm = 'cp ' +fpah     + ' ./Input/PAH2170.wq'
                  spawn, comm
; print, '2:   ', comm

; read parameters and reddening curve from idl saveset file fxdr:
                  restore, fxdr

                  
   tauVGaia  = vtauv(i) 

   model    = reddpolfit
   data     = data  * ebv_ref
   edata    = data  * ebv_ref  * 0.1
   wdata    = wdata
   data(0)  = -tauVGaia * 1.08574     ; set -Av from Gaia distance if not availabe ignore (set it ot -999.)
  edata(0)  = 1.d9                   ; ignore fitting data(0) for iter le 2


   
; save file jsmTaufit.inp which will be read by a.crossec_dd   
 get_lun, iu
 openw,iu,'./Input/jsmTaufit.inp'
 printf, iu, ' tauV, ebv = '
 printf,iu, tauVGaia, ebv_ref
 close, iu
 free_lun, iu
   
readcol, './Input/jsm12fit.inp', /sil, skipline=1, numli=1,  $
          abuc, abusi, abuvsi, abucvgr, abucpahs, qmrn, alec,  alesi, $
          arad_polmin_aC, arad_polmin_Si, arad_polmax, aled 
 abusi   = abusi(0)*1d0
 aled    = aled(0)*1d0
 qmrn    = qmrn(0)  > 2.
 qmrn    = qmrn(0)  < 4.

 aled    = aled < 3.10d-4
 
 print, ' Start with input parameters used by a.crossec_dd'
 print, 'abuc, abusi, abuvsi, abucvgr, abucpahs, qmrn, alec, alesi, arad_polmin_aC, arad_polmin_Si, arad_polmax, aled, x0, gam'
 
 print, format='(6f8.3, 8e10.3)', abuc, abusi, abuvsi, abucvgr, abucpahs, qmrn, $
        alec, alesi, arad_polmin_aC, arad_polmin_Si, arad_polmax, aled, x0, gam  

; 
; ---------------------- now call MPFIT  --------------------------
; for iter le 2 ignore fitting data(0) = -Av by setting edatata(0)=1d9
; for iter gt 2 the reddening curve up to JHK is fit and then use this
; fit to estimate Av that is then used to re-do the comlpete reddening
; curve fit
; a) first by keeping radii free and abundances fixed
; b) second keep radii fixed and abundances free c) iterate 
 
     xtol = 1d-3
 for iter = 0, 5 do begin

 print
 print, '  MP1: free: lec, lesi, led  fixed: rest          at iter = ', iter
 print

 
 if iter le 3 then begin
      data_mp =  data(1:*)
     wdata_mp = wdata(1:*)
     edata_mp = edata(1:*)
     print, ' --------------------------------------------------------------- '
     print, ' iter<2: ignore E(oo)=-tauv in redd fit'
     print, '         largest redd wavelength fitted wdata_mp(0), data_mp(0): ',    wdata_mp(0), data_mp(0)
     print
 endif else begin 
     spawn,   './a.crossec_dd | tee  message.out'
     res      = -9
     spawn, 'grep tauV_dark message.out', res
     tauV       = (float(strmid(res, 43,9)))(0)
     tauV_Dark  = (float(strmid(res, 52,9)))(0)
     data(0)   = -tauV    * 1.08574
     edata(0)  = 0.1*tauv * 1.08574
      data_mp =  data(0:*)
     wdata_mp = wdata(0:*)
     edata_mp = edata(0:*)
     print
     print, ' --------------------------------------------------------------- '
     print, ' iter>1: set tauV from present reddening curve fit and set data=-tauv, edata = 0.1*tauv '
     print, '         largest redd wavelength fitted wdata_mp(0), -data_mp(0): ',   wdata_mp(0), -data_mp(0)
     print, '         1.0856 * [tauVGaia, tauv, err] = ', 1.08574 * [tauVGaia, tauv, tauVGaia/tauv]
     print
     
 endelse

 
 delvar, paRadius, parad
  paRadius = replicate({fixed:0, limited:[1,1], limits:[1,narad-1],mpside:2, step:1},3)
  paRadius(0).limits(0)  = 1.
  paRadius(0).limits(1)  = 17.
  paRadius(1).limits(0)  = 1.
  paRadius(1).limits(1)  = 17.
  paRadius(2).limits(0)  = 1.
  paRadius(2).limits(1)  = float(narad)-1.

  alec                   = alec(0)   < 3.28e-5
  alesi                  = alesi(0)  < 3.28e-5
  aled                   = aled(0)   < 3.10e-4
  aled                   = aled      < 3.10d-4

  lec                    = (where (rarad ge alec))(0)
  lesi                   = (where (rarad ge alesi))(0)
  led                    = (where (rarad ge aled))(0)
  parad         = float([lec,lesi,led])
  parad(0)      = ((parad(0) > paradius(0).limits(0)) < paradius(0).limits(1)) 
  parad(1)      = ((parad(1) > paradius(1).limits(0)) < paradius(1).limits(1))
  parad(2)      = ((parad(2) > paradius(2).limits(0)) < paradius(2).limits(1))
    


  if no_mpfit ge 1 then begin
     res = parad
  endif else begin
     stat = -1
     delvar, res
     res  = MPFITFUN('fkt_absreddRad', wdata_mp, data_mp, edata_mp, $
                 parad, PARINFO=paradius, perror=rms_para, status=stat,    $
                  yfit=reddfit, bestnorm=chi2, dof=dof, xtol = 1.)
     redchi2     = chi2/dof
  endelse
  


  alec        = rarad(nint(res(0))) < 3.28e-5 
  alesi       = rarad(nint(res(1))) < 3.28e-5 
  aled        = rarad(nint(res(2))) < 3.10e-4 
  aled        = aled                < 3.10d-4
  
  alec        = alec(0)   &   alesi   = alesi(0)  &   aled  = aled(0)

  get_lun, iu
  openw,iu,'./Input/jsm12fit.inp'
printf, iu, '  abuc    abusi    avsi   avgr    apah     qmrn  rlec      rlesi     r_pmin_aC r_pmin_Si r_pmax    r_Darkmax '
printf, iu, format='(6f8.3, 6e10.3)', abuc, abusi, abuvsi, abucvgr, abucpahs, qmrn, $
        alec, alesi, arad_polmin_aC, arad_polmin_Si, arad_polmax, aled 
  close, iu
  free_lun, iu


  print, format='(6f8.3, 8e10.3)', abuc, abusi, abuvsi, abucvgr, abucpahs, qmrn, $
         alec, alesi, arad_polmin_aC, arad_polmin_Si, arad_polmax, aled, x0, gam


; ---------------------

  print
  print, '  MP2 :  with 12 parameters:'

;  print, ' 0        1        2      3       4        5     6         7         8         9         10        11       '
;print, '  abuc    abusi    avsi   avgr    apah     qmrn  rlec      rlesi     r_pmin_aC r_pmin_Si r_pmax    r_Darkmax '
   

  alec        = alec(0)   &   alesi   = alesi(0)  &   aled   = aled(0) 
  para        = [abuc(0), abuvsi(0), abucvgr(0), abucpahs(0), qmrn(0),    $
                 alec(0), alesi(0), arad_polmin_aC(0), arad_polmin_Si(0), $
                 x0, gam]  * 1.d0
  


delvar, parain
parain = replicate({fixed:0, limited:[1,1], limits:[0.D,0.D],mpside:2},11)
parain(0).limits(0) =   1d0
parain(0).limits(1) = 160.0d0
parain(1).limits(0) =   1.0d-2
parain(1).limits(1) =  20.0d0
parain(2).limits(0) =   1.0d-2
parain(2).limits(1) =  40.0d0
parain(3).limits(0) =   1.0d-2
parain(3).limits(1) =  40.0d0
parain(4).limits(0) =   2.d0         ; qmrn
parain(4).limits(1) =   4.d0         ; 
parain(5).limits(0) =  min(rarad)    ; alec
parain(5).limits(1) =  3.28e-5
parain(6).limits(0) =  min(rarad)    ; alesi
parain(6).limits(1) =  3.28e-5  ; 
parain(7).limits(0) =  5d-6          ; arad_polmin_aC
parain(7).limits(1) =  3.28e-5  ; 
parain(8).limits(0) =  5d-6          ; arad_polmin_Si
parain(8).limits(1) =  3.28e-5  ; 
parain(9).limits(0) =  4.3           ; PAH x0
parain(9).limits(1) =  4.9
parain(10).limits(0)=  0.5           ; PAH gam
parain(10).limits(1)=  2.0


print, '  MP2: free: abundance+qmrn, fixed: radii         at iter = ', iter
print

   parain.fixed      = 0
   parain(5:8).fixed = 1
   save, filename='./Input/tmp_parain.xdr', parain

  if no_mpfit ge 1 then begin
     res = para
  endif else begin
   stat = -1 
   delvar, res
   res  = MPFITFUN('fkt_absredd', wdata_mp, data_mp, edata_mp, $
                 para, PARINFO=parain, perror=rms_para, status=stat,    $
                 yfit=reddfit, bestnorm=chi2, dof=dof, xtol=xtol)
   redchi2     = chi2/dof
   para           = res
   
   rmsabuc           = rms_para(0)
   rmsabuvsi         = rms_para(1)
   rmsabucvgr        = rms_para(2)
   rmsabucpahs       = rms_para(3)
   rmsqmrn           = rms_para(4)
   rmsalec           = rms_para(5)
   rmsalesi          = rms_para(6)
   rmsarad_polmin_aC = rms_para(7)
   rmsarad_polmin_Si = rms_para(8)
   rmsx0             = rms_para(9)
   rmsgam            = rms_para(10)   
  endelse
  
   para(0)        = ((para(0) > parain(0).limits(0)) < parain(0).limits(1)) 
   para(1)        = ((para(1) > parain(1).limits(0)) < parain(1).limits(1)) 
   para(2)        = ((para(2) > parain(2).limits(0)) < parain(2).limits(1)) 
   para(3)        = ((para(3) > parain(3).limits(0)) < parain(3).limits(1)) 
   para(4)        = ((para(4) > parain(4).limits(0)) < parain(4).limits(1))
   para(5)        = ((para(5) > parain(5).limits(0)) < parain(5).limits(1))
   para(6)        = ((para(6) > parain(6).limits(0)) < parain(6).limits(1)) 
   para(7)        = ((para(7) > parain(7).limits(0)) < parain(7).limits(1))
   para(8)        = ((para(8) > parain(8).limits(0)) < parain(8).limits(1)) 
   para(7)        =   para(7) < para(5)
   para(8)        =   para(8) < para(6)
   para(9)        = ((para(9) > parain( 9).limits(0)) < parain( 9).limits(1))
   para(10)       = ((para(10)> parain(10).limits(0)) < parain(10).limits(1)) 

   abuc           =   para(0)
   abusi          =   abusi(0) *1d0
   abuvsi         =   para(1)
   abucvgr        =   para(2)
   abucpahs        =  para(3)
   qmrn           =   para(4)
   alec           =   para(5)
   alesi          =   para(6)
   arad_polmin_aC =   para(7)
   arad_polmin_Si =   para(8)   
   x0             =   para(9)
   gam            =   para(10)
   

 
get_lun, iu
openw,iu,'./Input/jsm12fit.inp'
printf, iu, '  abuc    abusi    avsi   avgr    apah     qmrn  rlec      rlesi     r_pmin_aC r_pmin_Si r_pmax    r_Darkmax '
printf, iu, format='(6f8.3, 6e10.3)', abuc, abusi, abuvsi, abucvgr, abucpahs, qmrn, $
        alec, alesi, arad_polmin_aC, arad_polmin_Si, arad_polmax, aled 
close, iu
free_lun, iu

; use org Drude parameters not those form redd fit !
   get_lun, iu
   openw,iu,'./Input/PAH2170.wq'
   printf, iu, 'x0 (mu)    gam : use ORG Drude Parameters of PAH at 2170 AA bump'
   printf,iu, x0, gam          ; change if ext papra shall be used
   printf,iu, x0org, gamorg    ; NOT used as OLD parameters as in ref paper 
   close, iu
   free_lun, iu
   

   if not keyword_set(ps) then begin
      spawn,   './a.crossec_dd | tee  message.out'
      res      = -9
      spawn, 'grep tauV_dark message.out', res
      tauV       = (float(strmid(res, 43,9)))(0)    
      rdfloat, './Output/Kappa.out', skipline=3, w, sac, ssac, sasi, $
        sssi, savgr, ssvgr, savsi, ssvsi, sapah, sad, ssd, st, /sil
        w   = w*1e4
        kvis  = (where(w le 0.548))(0) & kblue  = (where(w le 0.445))(0)
        absredd_mod  = 2.5/alog(10.) * (st/st(kvis) -1.) * st(kvis)
        yrange = [min([-1.08574*[tauVGaia, tauV], absredd_mod(0), data(0), edata(0)])-0.5,    max([data(0:*)+0.5])]
        
        plot, 1./w, absredd_mod, xrange=[-0.5,11.1], yrange=yrange, /xsty, /ysty
        oploterror, 1./wdata, data, wdata*0, edata, /nohat, color=fsc_color('dark gray'), psym=3
        oplot, 1./wdata(0) *[1,1],  [data(0), data(0)],  psym=plotsym(/box,scale=1.3,/fill,color=fsc_color('charcoal'))
        oplot, 1./wdata(0) *[1,1], -[tauVGaia,tauVGaia]*1.08574,  psym=plotsym(/circle,scale=1.3,color=fsc_color('green'), /fill)
        oplot, 1./wdata(0) *[1,1], -[tauV,tauV]*1.08574,  psym=plotsym(/box,scale=1.3,color=fsc_color('red'), /fill)
        oplot, 1./w, absredd_mod, thick=1, color=fsc_color('magenta')
        xyouts, 1, max([data*0.65]), target, charsize=1.5
        legend, ['Av(Gaia)', 'Av(E!DB-V)!N', 'E!DB-V!N*data(0)'], textcolor=[fsc_color('green'), fsc_color('red'), fsc_color('charcoal')]


  endif


 print, '   1.08574*[tauVGaia,   tauV],   -data(0),  -data_mp(0)'
 print, format='(f,f,f,f,f)', 1.08574*[tauVGaia, tauV], -data(0), -data_mp(0)
 print, format='(6f8.3, 8e10.3)', abuc, abusi, abuvsi, abucvgr, abucpahs, qmrn, $
        alec, alesi, arad_polmin_aC, arad_polmin_Si, arad_polmax, aled, x0, gam



endfor
;

  
 
 print
 print, ' ----------------------------  MPfit is done   -------------------'
 print



; ========================================================================
; finale creduced chi2

 redchi2     = chi2/dof

 
;check abundance constraints [C]/[Si] < 5.25
 if  abuc gt 5.25*(abusi + abuvsi) -abucvgr - abucpahs then stop, 'abundance constraint not ok nach mpfit'

 
; ===============================
; Save Fit results MPFIT results :
; ===============================

      

spawn,   './a.crossec_dd | tee  message.out'


res      = -9
spawn, 'grep Mass_mrn message.out', res
Mmrn   = float(strmid(res, 43,9))
Mdark  = float(strmid(res, 55,9))
fdark  = float(strmid(res, 67,9))
Mmrn   = Mmrn(0) & Mdark=Mdark(0) & fdark = fdark(0)  
print, 'Mmrn, Mdark, fdark'
print, Mmrn, Mdark, fdark

res      = -9
spawn, 'grep tauV_dark message.out', res
tauV       = (float(strmid(res, 43,9)))(0)
tauV_Dark  = (float(strmid(res, 52,9)))(0)




 print, '1.08574 *[tauVGaia, tauV], -data(0),  -data_mp(0)'
 print, format='(f,f,f,f,f)', 1.08574*[tauVGaia, tauV], -data(0), -data_mp(0)

; -------------------------  PLOT result ---------


; Kappa.out
rdfloat, './Output/Kappa.out', skipline=3, w, sac, ssac, sasi, $
        sssi, savgr, ssvgr, savsi, ssvsi, sapah, sad, ssd, st, /sil
  w   = w*1e4
  kvis  = (where(w le 0.548))(0) & kblue  = (where(w le 0.445))(0)
  Rv_mod      = st(kvis) / (st(kblue) - st(kvis))
 Ebv_mod      = 2.5/alog(10.) * (st(kblue) - st(kvis))
  Av_mod      = Rv_mod * Ebv_mod
 absredd_mod  = 2.5/alog(10.) * (st/st(kvis) -1.) * st(kvis) 
 Rv_ref       = vRv_ref(i)
 print, ' w(kvis), w(kblue)     : ',  w(kvis), w(kblue)
 print, ' Ebv_ref, Ebv_mod : ', ebv_ref, ebv_mod
 print, ' Rv_ref,  Rv_mod  : ',  Rv_ref,  Rv_mod


    salec  = strmid(string(nint(alec*1e7)),  5, 7) 
    salesi = strmid(string(nint(alesi*1e7)), 5, 7)
    saled  = strmid(string(nint(aled*1e7)),  4, 7) 
    spc    = strmid(string(nint(arad_polmin_aC*1e7)), 5, 7) 
    spsi   = strmid(string(nint(arad_polmin_si*1e7)), 5, 7) 

    tit    = 'c2=' + strmid(string(nint(100*redchi2)/100.),4,4)   + $
             ' R!DC!N= '  + salec+ ' R!DSi!N= ' + salesi + ' R!DD!N= ' + saled + $
             ' Rv:' +strmid(Rv_mod,6,4)+' d/l='+strmid(fdark,5,4)
    
    yrange = [min([-1.08574*[tauVGaia, tauV], absredd_mod(0), data(0), edata(0)])-0.5,    max([data(0:*)+0.5])]

 plot, 1./w, absredd_mod, xrange=[-0.5,11.1], yrange=yrange, /xsty, /ysty, $
       xthick=4, ythick=4, /nodata
 
 oplot, 1./w, ((sac+ssac + sasi+sssi)/st(kvis)   -1.)* Av_mod, color=fsc_color('brown')
 oplot, 1./w, ((savsi+ssvsi+ sapah+savgr+ssvgr)/st(kvis) -1.)* Av_mod, color=fsc_color('forest green')
 oplot, 1./w, ((sad+ssd)/st(kvis)     -1.)            * Av_mod, thick=!P.thick+1
 oplot, 1./w, absredd_mod, thick=4, color=fsc_color('magenta')




; data
 oploterror, 1./wdata, data, wdata*0, 0.1*abs(data) > 0.05, /nohat, $
             color=fsc_color('dark gray'), psym=3
 oplot, 1./wdata(1:*), data(1:*),  psym=plotsym(/circle,scale=1.01,color=fsc_color('gray'), /fill)
 oplot, 1./wdata(1:*), data(1:*),  psym=plotsym(/circle,scale=1.01)

; model if tauV modified from Dgaia:
 oplot, 1./wdata(0) *[1,1], [data(0), data(0)],  psym=plotsym(/box,scale=1.5,color=fsc_color('Charcoal'), /fill)
 oplot, 1./wdata(0) *[1,1], -1.08574*[tauVGaia,tauVGaia],  psym=plotsym(/circle,scale=1.3,color=fsc_color('green'), /fill)
 oplot, 1./wdata(0) *[1,1], -1.08574*[tauV,tauV],  psym=plotsym(/box,scale=1.,color=fsc_color('red'), /fill)


 
 oplot, 1./w, absredd_mod, thick=1, color=fsc_color('magenta')
 xyouts, 1, max([data*0.65]), target, charsize=1.5


  legend, ['Av(Gaia)', 'Av(E!DB-V)!N', 'E!DB-V!N*data(0)'], textcolor=[fsc_color('green'), fsc_color('red'), fsc_color('charcoal')]


 print, 'tauVGaia, tauV, -data(0),  -data_mp(0)'
 print, format='(f,f,f,f,f)', 1.08574*[tauVGaia, tauV], -data(0), -data_mp(0)

 
; ======================
; Save Fit results 
; =================


 data (0) = -tauVGaia * 1.08574
 edata(0) = 0.1 * data(0)
 data     = data / ebv_ref           ; save relatkive reddening with data() from Gaa estimate


print, ' in xdr save file =', './ResultEbv/'+ target+'_para.xdr' 
print, ' relative reddening E(x-V)/Ebv is saved'

    save, /xdr, filename='./ResultEbv/'+ target+'_para.xdr', $
     target, redchi2, redchi2p, $ 
     abuc, abusi,  abuvsi,    abucvgr,    abucpahs,  qmrn, alec, alesi, aled, $
          arad_polmin_aC, arad_polmin_Si, arad_polmax, x0, gam, $
           Mmrn, Mdark, fdark, $
      Av_ref, Rv_ref,  Ebv_ref, tauVgaia, tauV, tauV_Dark, $
      wredd_org, redd_org, eredd_org, wdata, data, edata, reddpolfit, $ 
      w, sac, ssac, sasi, sssi, savgr, ssvgr, savsi, ssvsi, sapah, sad, ssd, st
    
 
 spawn, 'cp ./Output/Kappa.out    ./ResultEbv/'+ target+'_Kappa.out'
 spawn, 'cp ./Output/PolKappa.out ./ResultEbv/'+ target+'_PolKappa.out'
 spawn, 'cp ./Input/jsm12fit.inp  ./ResultEbv/'+ target+'_jsm12fit.inp'
 spawn, 'cp ./Input/PAH2170.wq    ./ResultEbv/'+ target+'_PAH2170.wq'
 spawn, 'mv message.out           ./ResultEbv/'+ target+'_message.out'



 print, ' ***  Target: tauv(Gaia), tauv(Ebv) = ', target, tauVGaia, tauv

endif        ; target=star(i)
endfor       ; loop over all stars
   

if keyword_set(ps) then begin
   device, /close
   set_plot, 'X'
   device, decomposed=1
   print
   print
   print, ' ResultEbv of reddening fit in plot : pl_allRedd_Ebv.pdf'
   com =  ' ps2pdf                  idl.ps       pl_allRedd_Ebv.pdf'
   spawn, com
   print, com + '     ->      DONE'
endif

print
print, ' mp_absredd: DONE'
print

end
